/***************************************************************************
* File Name: hal430_RS232Tx.c
* Processor(s): MSP430F16x (tested), MSP430F14x (not tested)
* Compiler: Code Composer Studio V4 (4.2.5.00005)
* 
* Copyright  2012 Playing With Fusion, Inc.
* SOFTWARE LICENSE AGREEMENT: This code is released under the MIT License. 
* 
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
* DEALINGS IN THE SOFTWARE.
* **************************************************************************
* REVISION HISTORY:
* Author			Date		Comments
* J. Steinlage		2012Dec16	Original version
* 
* **************************************************************************
* ADDITIONAL NOTES:
* This file contains functions initialize and broadcast formatted data
* to the UART pins configured for this application note.
* 	- Configure UART for 38.4kBaud based on a ~4.9MHz DCO frequency
* 	- Broadcast characters or strings of characters the the UART
* 	- Format and broadcast temperature data from the MAX31855K-4CH board
***************************************************************************/

#include "msp430x16x.h"
#include "hal430_RS232Tx.h"
#include "stdio.h"
#include "pwf_an0001_defines.h"
#include "msp430_typeDef.h"
#include "pwf_an0001_struct.h"

extern struct	var_signed16 temp_TC0;
extern struct	var_signed16 temp_TC1;
extern struct	var_signed16 temp_TC2;
extern struct	var_signed16 temp_TC3;

// character strings for UART broadcast headings
static char TC_TEMP0_STR[] 		= "TC 0: ";
static char TC_TEMP1_STR[] 		= "TC 1: ";
static char TC_TEMP2_STR[] 		= "TC 2: ";
static char TC_TEMP3_STR[] 		= "TC 3: ";
static char TC_SNSR_OPEN[]		= "OPEN    ";
static char TC_SNSR_SVCC[]		= "S2VCC   ";
static char TC_SNSR_SGND[]		= "S2GND   ";
static char TC_SNSR_FUNK[]		= "F_UNK   ";

void RS232_Tx_init(void)	// Modified for use on Oven Controller V2.0
{
	// UART Clock set to SMCLK(DCO = 4.9 MHz)
	U1CTL = SWRST;								// U1CTL SWRST 
	ME2 |= UTXE1;								// Enable USART1 TXD
	U1CTL |= CHAR;								// 8-bit character
	U1TCTL = SSEL0 + SSEL1;						// BRCLK = SMCLK
	U1BR0 = 0x7F;								// ~4.9 MHz SMCLK/ 127.6  = 38400 baud
	U1BR1 = 0x00;								//
	U1MCTL = 0xB5;								// Modulation for 4.9 MHz SMCLK, 38400 baud
	U1CTL &= ~SWRST;							// Initialize USART state machine
}

void RS232_putchar(unsigned char data)
{
	while (!(U1IFG & UTXIFG1));                  // wait for Tx buffer to be empty - transmit when Tx buffer empty
	U1TXBUF = data;
}

void RS232_putCharStr(char *data)
{
  while(*data)
  {
	RS232_putchar(*data++);
  }
}

void RS232_disable(void)
{
  while(0x01 & ~U1TCTL)                     // wait until TXEPT is set (1)
  UCTL1 = SWRST;
}

void RS232_return(void)
{
  while(!(U1IFG & UTXIFG1));
  U1TXBUF = 0x0D;
}

void RS232_tab(void)
{
  while(!(U1IFG & UTXIFG1));
  U1TXBUF = 0x09;
}

void uart_single_ch_write(struct var_signed16 *TC_ptr)
{
	unsigned short frac, dec;
	unsigned int size;
	char conv_to_str[8];
	
	if(0 == TC_ptr->status)					// Thermocouple input not faulted
	{
		frac = (TC_ptr->value&0x03);			// find fraction portion of temperature (C)
		dec = (TC_ptr->value>>2);				// Right-shift decimal portion to get proper value
		size = sprintf(conv_to_str, "%d", dec);	// TC temp to string
		RS232_putCharStr(conv_to_str);			// Write decimal portion of temp to UART (char string)
		//write fraction portion (1/4 degC per bit)
		if(0 == frac){RS232_putCharStr(".00");}
		else if(1 == frac){RS232_putCharStr(".25");}
		else if(2 == frac){RS232_putCharStr(".50");}
		else{RS232_putCharStr(".75");}
		RS232_putCharStr("C ");				// write deg C symbol and spacing
	}
	else if(1 == TC_ptr->status){RS232_putCharStr(TC_SNSR_OPEN);}		// Thermocouple input open
	else if(2 == TC_ptr->status){RS232_putCharStr(TC_SNSR_SGND);}		// Thermocouple input short to GND
	else if(3 == TC_ptr->status){RS232_putCharStr(TC_SNSR_SVCC);}		// Thermocouple input short to VCC
	else{RS232_putCharStr(TC_SNSR_FUNK);}								// Unknown TC input fault
}

void update_uart_250ms(void)
{
	// Broadcast temperatures to UART with headings formatted for direct
	// reading via 'hyperterminal' or 'putty'
	
	// This code broadcasts ONLY the thermocouple data. It does not broadcast ref jcn data
	// All results are in deg C
	struct var_signed16 *tc_ch_ptr;
	// TC0
	RS232_putCharStr(TC_TEMP0_STR);
	tc_ch_ptr = &temp_TC0;
	uart_single_ch_write(tc_ch_ptr);
	// TC1
	RS232_putCharStr(TC_TEMP1_STR);
	tc_ch_ptr = &temp_TC1;
	uart_single_ch_write(tc_ch_ptr);
	// TC2
	RS232_putCharStr(TC_TEMP2_STR);
	tc_ch_ptr = &temp_TC2;
	uart_single_ch_write(tc_ch_ptr);
	// TC3
	RS232_putCharStr(TC_TEMP3_STR);
	tc_ch_ptr = &temp_TC3;
	uart_single_ch_write(tc_ch_ptr);
	
	RS232_return();
	
////	// This code broadcasts reference jcn temp for each thermocouple in deg C
////	unsigned int size;
////	char conv_to_str[8];
////	unsigned int ref_tmp;
////
////	RS232_putCharStr("JCN0: ");
////	ref_tmp = temp_TC0.ref_jcn_temp & ~0x8000;		// get rid of sign bit
////	ref_tmp = ref_tmp >> 4;							// drop fraction
////	if(0x8000 == temp_TC0.ref_jcn_temp & 0x8000){ref_tmp |= 0x8000;}	// add sign back
////	size = sprintf(conv_to_str, "%d", ref_tmp);		// TC temp to string
////	RS232_putCharStr(conv_to_str);					// Write decimal portion of temp to UART (char string)
////	RS232_putCharStr("C    ");						// add spaces for formatting
////	
////	RS232_putCharStr("JCN1: ");
////	ref_tmp = temp_TC1.ref_jcn_temp & ~0x8000;		// get rid of sign bit
////	ref_tmp = ref_tmp >> 4;							// drop fraction
////	if(0x8000 == temp_TC1.ref_jcn_temp & 0x8000){ref_tmp |= 0x8000;}	// add sign back
////	size = sprintf(conv_to_str, "%d", ref_tmp);		// TC temp to string
////	RS232_putCharStr(conv_to_str);					// Write decimal portion of temp to UART (char string)
////	RS232_putCharStr("C    ");						// add spaces for formatting
////	
////	RS232_putCharStr("JCN2: ");
////	ref_tmp = temp_TC2.ref_jcn_temp & ~0x8000;		// get rid of sign bit
////	ref_tmp = ref_tmp >> 4;							// drop fraction
////	if(0x8000 == temp_TC2.ref_jcn_temp & 0x8000){ref_tmp |= 0x8000;}	// add sign back
////	size = sprintf(conv_to_str, "%d", ref_tmp);		// TC temp to string
////	RS232_putCharStr(conv_to_str);					// Write decimal portion of temp to UART (char string)
////	RS232_putCharStr("C    ");						// add spaces for formatting
////	
////	RS232_putCharStr("JCN3: ");
////	ref_tmp = temp_TC3.ref_jcn_temp & ~0x8000;		// get rid of sign bit
////	ref_tmp = ref_tmp >> 4;							// drop fraction
////	if(0x8000 == temp_TC3.ref_jcn_temp & 0x8000){ref_tmp |= 0x8000;}	// add sign back
////	size = sprintf(conv_to_str, "%d", ref_tmp);		// TC temp to string
////	RS232_putCharStr(conv_to_str);					// Write decimal portion of temp to UART (char string)
////	RS232_putCharStr("C    ");						// add spaces for formatting
////
////	RS232_return();
	
	
//  Code below just dumps raw values to UART... would need to be interpreted by something
//	RS232_putCharStr(RES_250MS_SOF);						// Header signifying start of new data
//	temp = temp_TC0.value >> 8;								// high byte of TC0.value
//	RS232_putchar(temp);
//	temp = temp_TC0.value & 0xff;							// low byte of TC0.value
//	RS232_putchar(temp);
//	RS232_putchar(temp_TC0.status);							// TC0.status
//	temp = temp_TC1.value >> 8;								// high byte of TC1.value
//	RS232_putchar(temp);
//	temp = temp_TC1.value & 0xff;							// low byte of TC1.value
//	RS232_putchar(temp);
//	RS232_putchar(temp_TC1.status);							// TC1.status
//	temp = temp_TC2.value >> 8;								// high byte of TC2.value
//	RS232_putchar(temp);
//	temp = temp_TC2.value & 0xff;							// low byte of TC2.value
//	RS232_putchar(temp);
//	RS232_putchar(temp_TC2.status);							// TC2.status
//	temp = temp_TC3.value >> 8;								// high byte of TC3.value
//	RS232_putchar(temp);
//	temp = temp_TC3.value & 0xff;							// low byte of TC3.value
//	RS232_putchar(temp);
//	RS232_putchar(temp_TC3.status);							// TC3.status

}
